searchNodes=[{"ref":"PhoenixApiToolkit.html","title":"PhoenixApiToolkit","type":"module","doc":"Documentation for PhoenixApiToolkit."},{"ref":"PhoenixApiToolkit.Ecto.DynamicFilters.html","title":"PhoenixApiToolkit.Ecto.DynamicFilters","type":"module","doc":"Dynamic filtering of Ecto queries is useful for creating list/index functions, and ultimately list/index endpoints, that accept a map of filters to apply to the query. Such a map can be based on HTTP query parameters, naturally. This module complements PhoenixApiToolkit.Ecto.GenericQueries by leveraging the generic queries provided by that module to filter a query dynamically based on a parameter map. Several filtering types are so common that they have been implemented using standard filter macro&#39;s. This way, you only have to define which fields are filterable in what way. Example without standard filters def list_without_standard_filters(filters \\\\ %{}) do from(user in &quot;users&quot;, as: :user) |&gt; apply_filters(filters, fn {:order_by, {field, direction}}, query -&gt; GenericQueries.order_by(query, :user, field, direction) {literal, value}, query when literal in [:id, :name, :residence, :address] -&gt; GenericQueries.equals(query, :user, literal, value) _, query -&gt; query end) end # filtering is optional iex&gt; list_without_standard_filters() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; # multiple literal matches can be combined iex&gt; list_without_standard_filters(%{residence: &quot;New York&quot;, address: &quot;Main Street&quot;}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.address == ^&quot;Main Street&quot;, where: u0.residence == ^&quot;New York&quot;&gt; # literal matches and sorting can be combined iex&gt; list_without_standard_filters(%{residence: &quot;New York&quot;, order_by: {:name, :desc}}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.residence == ^&quot;New York&quot;, order_by: [desc: u0.name]&gt; # other fields are ignored / passed through iex&gt; list_without_standard_filters(%{number_of_arms: 3}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; Example with standard filters with module attributes The easiest to use and recommended form of standard filtering is the standard_filters/2 macro, which reads several module attributes from the module in which it is used to provide its functionality. @main_binding :user @literals ~w(id username residence address)a @sets ~w(roles)a @smaller_than_map %{ inserted_before: :inserted_at, updated_before: :updated_at } @smaller_than Map.keys(@smaller_than_map) @greater_than_or_equals_map %{ inserted_at_or_after: :inserted_at, updated_at_or_after: :updated_at } @greater_than_or_equals Map.keys(@greater_than_or_equals_map) def by_username_prefix(query, prefix) do from(user in query, where: ilike(user.username, ^&quot;\#{prefix}%&quot;)) end def list_with_standard_filters_and_attributes(filters \\\\ %{}) do from(user in &quot;users&quot;, as: :user) |&gt; apply_filters(filters, fn # Add custom filters first and fallback to standard filters {:username_prefix, value}, query -&gt; by_username_prefix(query, value) filter, query -&gt; standard_filters(query, filter) end) end # filtering is optional iex&gt; list_with_standard_filters_and_attributes() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; # filtering works the same way iex&gt; list_with_standard_filters_and_attributes(%{username: &quot;Peter&quot;, inserted_before: DateTime.from_unix!(155555555)}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.inserted_at &lt; ^~U[1974-12-06 09:52:35Z], where: u0.username == ^&quot;Peter&quot;&gt; # limit, offset, and order_by are supported iex&gt; list_with_standard_filters_and_attributes(%{limit: 10, offset: 1, order_by: {:username, :desc}}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, order_by: [desc: u0.username], limit: ^10, offset: ^1&gt; # complex custom filters can be user too iex&gt; list_with_standard_filters_and_attributes(%{username_prefix: &quot;Pete&quot;}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: ilike(u0.username, ^&quot;Pete%&quot;)&gt; # other fields are ignored / passed through iex&gt; list_with_standard_filters_and_attributes(%{number_of_arms: 3, order_by: {:boom, :asc}}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; Example with standard filters It is possible to use the standard filters macro without using module attributes, by specifying (some of) the macro parameters directly. def list_with_standard_filters(filters \\\\ %{}) do from(user in &quot;users&quot;, as: :user) |&gt; apply_filters(filters, fn filter, query -&gt; standard_filters( query, filter, :user, [:username], [:roles], @smaller_than_map, @smaller_than, @greater_than_or_equals_map, @greater_than_or_equals ) end) end # filtering is optional iex&gt; list_with_standard_filters() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; # filtering works the same way iex&gt; list_with_standard_filters(%{username: &quot;Peter&quot;}) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.username == ^&quot;Peter&quot;&gt;"},{"ref":"PhoenixApiToolkit.Ecto.DynamicFilters.html#apply_filters/3","title":"PhoenixApiToolkit.Ecto.DynamicFilters.apply_filters/3","type":"function","doc":"Applies filters to query by reducing filters using filter_reductor. Combine with the generic queries from PhoenixApiToolkit.Ecto.GenericQueries to write complex filterables. Several standard filters have been implemented in standard_filters/2 and standard_filters/9. See the module docs Elixir.PhoenixApiToolkit.Ecto.DynamicFilters for details and examples."},{"ref":"PhoenixApiToolkit.Ecto.DynamicFilters.html#standard_filters/2","title":"PhoenixApiToolkit.Ecto.DynamicFilters.standard_filters/2","type":"macro","doc":"Applies standard filters to the query. See apply_filters/3 for an example. Standard filters include filters for literal matches, datetime relatives, set membership, ordering and pagination. See the module docs Elixir.PhoenixApiToolkit.Ecto.DynamicFilters for details and examples. This macro requires that the following module attributes have been set: @main_binding: the named binding of the Ecto model that generic queries are applied to @literals: fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.equals/4 @sets: fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.member_of/4 @smaller_than_map: map of virtual &quot;smallerthan&quot; fields and the actual fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.smaller_than/4 @smaller_than: keys of @smaller_than_map @greater_than_or_equals_map: map of virtual &quot;greaterthan_or_equals&quot; fields and the actual fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.greater_than_or_equals/4 @greater_than_or_equals: keys of @greater_than_or_equals_map If these module attributes cannot be used, please use the fully parameterized version of this macro, standard_filters/9."},{"ref":"PhoenixApiToolkit.Ecto.DynamicFilters.html#standard_filters/9","title":"PhoenixApiToolkit.Ecto.DynamicFilters.standard_filters/9","type":"macro","doc":"Applies standard filters to the query. See apply_filters/3 for an example. Standard filters include filters for literal matches, datetime relatives, set membership, ordering and pagination. See the module docs Elixir.PhoenixApiToolkit.Ecto.DynamicFilters for details and examples. This macro requires the following parameters: main_binding: the named binding of the Ecto model that generic queries are applied to literals: fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.equals/4 sets: fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.member_of/4 smaller_than_map: map of virtual &quot;smallerthan&quot; fields and the actual fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.smaller_than/4 smaller_than: keys of smaller_than_map greater_than_or_equals_map: map of virtual &quot;greaterthan_or_equals&quot; fields and the actual fields comparable by PhoenixApiToolkit.Ecto.GenericQueries.greater_than_or_equals/4 greater_than_or_equals: keys of greater_than_or_equals_map"},{"ref":"PhoenixApiToolkit.Ecto.DynamicFilters.html#t:filter/0","title":"PhoenixApiToolkit.Ecto.DynamicFilters.filter/0","type":"type","doc":""},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html","title":"PhoenixApiToolkit.Ecto.GenericQueries","type":"module","doc":"Generic queries are applicable to any named binding in a query. By using generic queries, it is not necessary to implement standard queries for every Ecto model. For example, instead of implementing in a User model: def by_username(query, username) do from [user: user] in query, where: user.username == ^username end User.by_username(query, &quot;some username&quot;) ...you can use generic query equals/4 instead: GenericQueries.equals(query, :user, :username, &quot;some username&quot;) Such generic queries can be combined together in complex ways: iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; base_query() ...&gt; |&gt; equals(:user, :name, &quot;Peter&quot;) ...&gt; |&gt; smaller_than(:user, :inserted_at, DateTime.from_unix!(1555523215)) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.name == ^&quot;Peter&quot;, where: u0.inserted_at &lt; ^~U[2019-04-17 17:46:55Z]&gt; Most of these generic queries rely on named bindings to do their work. That&#39;s why it&#39;s probably a good idea to always name all bindings in your queries, and not rely on positional bindings to separate models in your queries."},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#equals/4","title":"PhoenixApiToolkit.Ecto.GenericQueries.equals/4","type":"function","doc":"Narrow down the query to results in which the value of binding.field is equal to value. If value is a list, results that are equal to any list element are returned. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; equals(base_query(), :user, :name, &quot;Peter&quot;) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.name == ^&quot;Peter&quot;&gt; iex&gt; equals(base_query(), :user, :name, [&quot;Peter&quot;, &quot;Patrick&quot;]) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.name in ^[&quot;Peter&quot;, &quot;Patrick&quot;]&gt;"},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#greater_than_or_equals/4","title":"PhoenixApiToolkit.Ecto.GenericQueries.greater_than_or_equals/4","type":"function","doc":"Narrow down the query to results in which the value contained in binding.field is greater than or equal to value. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; greater_than_or_equals(base_query(), :user, :inserted_at, DateTime.from_unix!(1555523215)) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.inserted_at &gt;= ^~U[2019-04-17 17:46:55Z]&gt;"},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#limit/2","title":"PhoenixApiToolkit.Ecto.GenericQueries.limit/2","type":"function","doc":"Limit the query result set size to value. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; limit(base_query(), 10) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, limit: ^10&gt;"},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#member_of/4","title":"PhoenixApiToolkit.Ecto.GenericQueries.member_of/4","type":"function","doc":"Narrow down the query to results in which value is a member of the set of values contained in field.binding. Use with array-type Ecto fields. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; member_of(base_query(), :user, :roles, &quot;admin&quot;) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: ^&quot;admin&quot; in u0.roles&gt;"},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#offset/2","title":"PhoenixApiToolkit.Ecto.GenericQueries.offset/2","type":"function","doc":"Offset the query results by value. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; offset(base_query(), 10) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, offset: ^10&gt;"},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#order_by/4","title":"PhoenixApiToolkit.Ecto.GenericQueries.order_by/4","type":"function","doc":"Order the query by binding.field in direction. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; order_by(base_query(), :user, :name, :asc_nulls_first) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, order_by: [asc_nulls_first: u0.name]&gt;"},{"ref":"PhoenixApiToolkit.Ecto.GenericQueries.html#smaller_than/4","title":"PhoenixApiToolkit.Ecto.GenericQueries.smaller_than/4","type":"function","doc":"Narrow down the query to results in which the value contained in binding.field is smaller than value. iex&gt; base_query() #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user&gt; iex&gt; smaller_than(base_query(), :user, :inserted_at, DateTime.from_unix!(1555523215)) #Ecto.Query&lt;from u0 in &quot;users&quot;, as: :user, where: u0.inserted_at &lt; ^~U[2019-04-17 17:46:55Z]&gt;"}]